@@ -215,7 +215,7 @@ class IEntityClass
 public:
 
     /// Signal emitted when entity class contents are changed or reloaded
-    virtual sigc::signal<void> changedSignal() const = 0;
+    virtual sigc::signal<void>& changedSignal() = 0;
 
     /// Get the name of this entity class
     virtual std::string getName() const = 0;
@@ -248,6 +248,9 @@ public:
     /// Return the display colour of this entity class
     virtual const Vector3& getColour() const = 0;
 
+    // Overrides the colour defined in the .def files
+    virtual void setColour(const Vector3& colour) = 0;
+
     /**
      * Get the named Shader used for rendering this entity class in
      * wireframe mode.
@@ -309,6 +312,9 @@ public:
 	 * given entity class name. className is treated case-sensitively.
 	 */
 	virtual bool isOfType(const std::string& className) = 0;
+
+    // Returns the mod-relative path to the file this DEF was declared in
+    virtual std::string getDefFileName() = 0;
 };
 
 /**
@@ -384,6 +390,8 @@ class IEntityClassManager :
     public RegisterableModule
 {
 public:
+    /// Signal emitted when all DEFs have been loaded (after module initialisation)
+    virtual sigc::signal<void> defsLoadedSignal() const = 0;
 
     /// Signal emitted when all DEFs are reloaded
     virtual sigc::signal<void> defsReloadedSignal() const = 0;
@@ -441,12 +449,8 @@ public:
  *
  * \ingroup eclass
  */
-inline IEntityClassManager& GlobalEntityClassManager() {
-    // Cache the reference locally
-    static IEntityClassManager& _eclassMgr(
-        *std::static_pointer_cast<IEntityClassManager>(
-            module::GlobalModuleRegistry().getModule(MODULE_ECLASSMANAGER)
-        )
-    );
-    return _eclassMgr;
+inline IEntityClassManager& GlobalEntityClassManager()
+{
+    static module::InstanceReference<IEntityClassManager> _reference(MODULE_ECLASSMANAGER);
+    return _reference;
 }
