@@ -1,8 +1,9 @@
 #pragma once
 
 #include "imodule.h"
+#include <sigc++/signal.h>
 
-const std::string MODULE_MAINFRAME("MainFrame");
+const char* const MODULE_MAINFRAME("MainFrame");
 
 class wxFrame;
 class wxToolBar;
@@ -107,16 +108,37 @@ public:
 	 */
 	virtual IScopedScreenUpdateBlockerPtr getScopedScreenUpdateBlocker(const std::string& title, 
 		const std::string& message, bool forceDisplay = false) = 0;
+
+	/**
+	 * A signal emitted when the MainFrame window has been set up. Modules can
+	 * subscribe to this to register any UI parts that require a valid main window
+	 * or sub component like the group dialog to be constructed.
+	 * This is a one-time signal, after emission the subscribers will be 
+	 * automatically removed by this class.
+	 */
+	virtual sigc::signal<void>& signal_MainFrameConstructed() = 0;
+
+	/**
+	 * Signal fired after the MainFrame window is shown the first time
+	 * during application start up.
+	 * This is a one-time signal, after emission the subscribers will be 
+	 * automatically removed by this class.
+	 */
+	virtual sigc::signal<void>& signal_MainFrameReady() = 0;
+
+	/**
+	 * Signal fired when the UI is shutting down, right before the MainFrame 
+	 * window will be destroyed. Dependant UI modules can listen to this
+	 * event to get a chance to clean up and save their state.
+	 * This is a one-time signal, after emission the subscribers will be
+	 * automatically removed by this class.
+	 */
+	virtual sigc::signal<void>& signal_MainFrameShuttingDown() = 0;
 };
 
 // This is the accessor for the mainframe module
 inline IMainFrame& GlobalMainFrame()
 {
-	// Cache the reference locally
-	static IMainFrame& _mainFrame(
-		*std::static_pointer_cast<IMainFrame>(
-			module::GlobalModuleRegistry().getModule(MODULE_MAINFRAME)
-		)
-	);
-	return _mainFrame;
+	static module::InstanceReference<IMainFrame> _reference(MODULE_MAINFRAME);
+	return _reference;
 }
