@@ -2,12 +2,19 @@
 
 #include "imodule.h"
 #include "inode.h"
+#include "imapexporter.h"
+#include "imapformat.h"
 #include "ikeyvaluestore.h"
 #include <sigc++/signal.h>
 
 // Registry setting for suppressing the map load progress dialog
 const char* const RKEY_MAP_SUPPRESS_LOAD_STATUS_DIALOG = "user/ui/map/suppressMapLoadDialog";
 
+// Whether to load the most recently used map on app startup
+const char* const RKEY_LOAD_LAST_MAP = "user/ui/map/loadLastMap";
+
+const char* const LOAD_PREFAB_AT_CMD = "LoadPrefabAt";
+
 // Namespace forward declaration
 class INamespace;
 typedef std::shared_ptr<INamespace> INamespacePtr;
@@ -41,6 +48,8 @@ class IMapRootNode :
 public:
     virtual ~IMapRootNode() {}
 
+	virtual void setName(const std::string& name) = 0;
+
     /**
      * greebo: Returns the namespace of this root.
      */
@@ -125,6 +134,38 @@ public:
 	* Returns the name of the map.
 	*/
 	virtual std::string getMapName() const = 0;
+
+	/**
+	 * Signal fired when the name of this map is changing.
+	 */
+	virtual sigc::signal<void>& signal_mapNameChanged() = 0;
+
+	// Returns true if the map has unsaved changes.
+	virtual bool isModified() const = 0;
+
+	// Sets the modified status of this map
+	virtual void setModified(bool modifiedFlag) = 0;
+
+	/**
+	 * Signal fired when the modified status of this map has changed.
+	 */
+	virtual sigc::signal<void>& signal_modifiedChanged() = 0;
+
+	// Caution: this is upposed to be called on startup, since it doesn't ask the user
+	// whether to save the current map. Use the "NewMap" command for regular purposes.
+	virtual void createNewMap() = 0;
+
+	// Create a MapExporter instance which can be used to export a scene,
+	// including the necessary preparation, info-file handling, etc.
+	// This is mainly a service method for external code, like the gameconnection.
+	virtual map::IMapExporter::Ptr createMapExporter(map::IMapWriter& writer, 
+		const scene::IMapRootNodePtr& root, std::ostream& mapStream) = 0;
+
+    // Exports the current selection to the given output stream, using the map's format
+    virtual void exportSelected(std::ostream& out) = 0;
+
+    // Exports the current selection to the given output stream, using the given map format
+    virtual void exportSelected(std::ostream& out, const map::MapFormatPtr& format) = 0;
 };
 typedef std::shared_ptr<IMap> IMapPtr;
 
@@ -133,12 +174,7 @@ const char* const MODULE_MAP("Map");
 // Application-wide Accessor to the currently active map
 inline IMap& GlobalMapModule()
 {
-	// Cache the reference locally
-	static IMap& _mapModule(
-		*std::static_pointer_cast<IMap>(
-			module::GlobalModuleRegistry().getModule(MODULE_MAP)
-		)
-	);
-	return _mapModule;
+	static module::InstanceReference<IMap> _reference(MODULE_MAP);
+	return _reference;
 }
 
