@@ -2,6 +2,7 @@
 
 #include "inode.h"
 #include "imodule.h"
+#include "itextstream.h"
 #include "imap.h"
 
 #include <sigc++/signal.h>
@@ -20,8 +21,25 @@ public:
 	// Renames this map resource to the new path
 	virtual void rename(const std::string& fullPath) = 0;
 
+	/**
+	 * Attempts to load the resource from disk. Returns true
+	 * on success, in which case the getRootNode() method can be
+	 * used to acquire a reference to the parsed map.
+	 * Will throw an OperationException on failure.
+	 */
 	virtual bool load() = 0;
 
+	// Exception type thrown by the the MapResource implementation
+	struct OperationException :
+		public std::runtime_error 
+	{
+		OperationException(const std::string& msg) :
+			runtime_error(msg)
+		{
+			rError() << "MapResource operation failed: " << msg << std::endl;
+		}
+	};
+
 	/**
 	* Save this resource
 	*
@@ -29,13 +47,14 @@ public:
 	* format argument is omitted, the format corresponding to the current
 	* game type is used.
 	*
-	* @returns
-	* true if the resource was saved, false otherwise.
+	* Throws an OperationException if anything prevented the save from
+	* completion (user cancellation, I/O errors)
 	*/
-	virtual bool save(const map::MapFormatPtr& mapFormat = map::MapFormatPtr()) = 0;
+	virtual void save(const map::MapFormatPtr& mapFormat = map::MapFormatPtr()) = 0;
 
-    virtual scene::IMapRootNodePtr getNode() = 0;
-    virtual void setNode(const scene::IMapRootNodePtr& node) = 0;
+    virtual const scene::IMapRootNodePtr& getRootNode() = 0;
+
+    virtual void clear() = 0;
 };
 typedef std::shared_ptr<IMapResource> IMapResourcePtr;
 
@@ -62,11 +81,6 @@ public:
 
 inline IMapResourceManager& GlobalMapResourceManager()
 {
-	// Cache the reference locally
-	static IMapResourceManager& _mapResourceManager(
-		*std::static_pointer_cast<IMapResourceManager>(
-			module::GlobalModuleRegistry().getModule(MODULE_MAPRESOURCEMANAGER)
-		)
-	);
-	return _mapResourceManager;
+	static module::InstanceReference<IMapResourceManager> _reference(MODULE_MAPRESOURCEMANAGER);
+	return _reference;
 }
