@@ -436,7 +436,7 @@ inline scene::INodePtr changeEntityClassname(const scene::INodePtr& node,
 	assert(eclass);
 
 	// Create a new entity with the given class
-	IEntityNodePtr newNode(GlobalEntityCreator().createEntity(eclass));
+	IEntityNodePtr newNode(GlobalEntityModule().createEntity(eclass));
 
 	Entity* oldEntity = Node_getEntity(oldNode);
 
@@ -476,3 +476,49 @@ inline scene::INodePtr changeEntityClassname(const scene::INodePtr& node,
 
 	return newNode;
 }
+
+/**
+ * greebo: This class can be used to traverse a subgraph to search
+ * for a specific spawnarg on the worldspawn entity. The method
+ * getValue() can be used to retrieve the value of the key as specified
+ * in the constructor.
+ */
+class WorldspawnArgFinder :
+    public scene::NodeVisitor
+{
+    std::string _key;
+    std::string _value;
+
+public:
+    WorldspawnArgFinder(const std::string& keyName) :
+        _key(keyName)
+    {}
+
+    bool pre(const scene::INodePtr& node) override
+    {
+        // Try to cast this node onto an entity
+        auto* ent = Node_getEntity(node);
+
+        if (ent != nullptr)
+        {
+            if (ent->isWorldspawn())
+            {
+                // Load the description spawnarg
+                _value = ent->getKeyValue(_key);
+            }
+
+            return false; // don't traverse entities
+        }
+
+        return true;
+    }
+
+    /**
+     * Returns the found value for the desired spawnarg. If not found,
+     * this function will return an empty string "".
+     */
+    const std::string& getFoundValue() const
+    {
+        return _value;
+    }
+};
