@@ -1,15 +1,16 @@
-#ifndef _CODE_TOKENISER_H_
-#define _CODE_TOKENISER_H_
+#pragma once
 
 #include "iarchive.h"
+#include "ifilesystem.h"
 #include "DefTokeniser.h"
 
 #include <list>
+#include <map>
+#include <algorithm>
+#include <fmt/format.h>
 
-#include <boost/algorithm/string/predicate.hpp>
-#include <boost/algorithm/string/trim.hpp>
-#include <boost/algorithm/string/replace.hpp>
-#include <boost/algorithm/string/classification.hpp>
+#include "string/trim.h"
+#include "string/predicate.h"
 #include "Tokeniser.h"
 
 namespace parser
@@ -74,7 +75,7 @@ public:
 		_keptDelims(keptDelims)
     {}
 
-    /* REQUIRED. Operator() is called by the boost::tokenizer. This function
+    /* REQUIRED. Operator() is called by the Tokeniser. This function
      * must search for a token between the two iterators next and end, and if
      * a token is found, set tok to the token, set next to position to start
      * parsing on the next call, and return true.
@@ -88,10 +89,18 @@ public:
         // Clear out the token, no guarantee that it is empty
         tok = "";
 
-        while (next != end) {
+		enum class QuoteType
+		{
+			Single = 0,
+			Double = 1,
+		};
 
-            switch (_state) {
+		QuoteType quoteType = QuoteType::Single;
 
+        while (next != end)
+		{
+            switch (_state)
+			{
                 case SEARCHING:
 
                     // If we have a delimiter, just advance to the next character
@@ -127,8 +136,8 @@ public:
 
                     // Now next is pointing at a non-delimiter. Switch on this
                     // character.
-                    switch (*next) {
-
+                    switch (*next) 
+					{
                         // Found a quote, enter QUOTED state, or return the
                         // current token if we are in the process of building
                         // one.
@@ -137,11 +146,23 @@ public:
                                 return true;
                             }
                             else {
+								quoteType = QuoteType::Double;
                                 _state = QUOTED;
                                 ++next;
                                 continue; // skip the quote
                             }
 
+						case '\'':
+							if (tok != "") {
+								return true;
+							}
+							else {
+								quoteType = QuoteType::Single;
+								_state = QUOTED;
+								++next;
+								continue; // skip the quote
+							}
+
                         // Found a slash, possibly start of comment
                         case '/':
                             _state = FORWARDSLASH;
@@ -240,7 +261,9 @@ public:
 
                     // In the quoted state, just advance until the closing
                     // quote. No delimiter splitting is required.
-                    if (*next == '\"') {
+                    if ((*next == '\"' && quoteType == QuoteType::Double) ||
+						(*next == '\'' && quoteType == QuoteType::Single))
+					{
                         ++next;
 
 						// greebo: We've found a closing quote, but there might be a backslash indicating
@@ -263,7 +286,11 @@ public:
 							{
 								tok += '\t';
 							}
-							else if (*next == '"') // Quote
+							else if (*next == '"' && quoteType == QuoteType::Double) // Escape Double Quote
+							{
+								tok += '"';
+							}
+							else if (*next == '\'' && quoteType == QuoteType::Single) // Escaped Single Quote
 							{
 								tok += '"';
 							}
@@ -320,7 +347,9 @@ public:
                         continue;
                     }
 
-					if (*next == '\"') {
+					if ((*next == '\"' && quoteType == QuoteType::Double) ||
+						(*next == '\'' && quoteType == QuoteType::Single))
+					{
 						// Found the desired opening quote, switch to QUOTED
 						++next;
 						_state = QUOTED;
@@ -428,30 +457,47 @@ public:
         else
             return false;
     }
+};
 
-    // REQUIRED. Reset function to clear internal state
-    void reset() {
-        _state = SEARCHING;
-    }
+// Represents a #DEFINE'd macro in a code file, which may have 0 to n arguments
+// #define MYDEF( y ) set "cmd" y;
+class Macro
+{
+public:
+	Macro()
+	{}
+
+	Macro(const std::string& name_) :
+		name(name_)
+	{}
+
+	// Name of the #DEFINE'd macro
+	std::string name;
+
+	// The arguments of this macro
+	std::list<std::string> arguments;
+
+	// The macro body
+	std::list<std::string> tokens;
 };
 
 class SingleCodeFileTokeniser :
 	public DefTokeniser
 {
+private:
     // Istream iterator type
     typedef std::istream_iterator<char> CharStreamIterator;
 
-    // Internal Boost tokenizer and its iterator
-    typedef boost::tokenizer<CodeTokeniserFunc,
-                             CharStreamIterator,
-                             std::string> CharTokeniser;
+    // Internal tokeniser and its iterator
+    typedef string::Tokeniser<CodeTokeniserFunc, CharStreamIterator> CharTokeniser;
     CharTokeniser _tok;
-    CharTokeniser::iterator _tokIter;
+    CharTokeniser::Iterator _tokIter;
 
 private:
 
 	// Helper function to set noskipws on the input stream.
-	static std::istream& setNoskipws(std::istream& is) {
+	static std::istream& setNoskipws(std::istream& is)
+	{
 		is >> std::noskipws;
 		return is;
 	}
@@ -479,7 +525,7 @@ public:
     : _tok(CharStreamIterator(setNoskipws(str)), // start iterator
            CharStreamIterator(), // end (null) iterator
            CodeTokeniserFunc(delims, keptDelims)),
-      _tokIter(_tok.begin())
+      _tokIter(_tok.getIterator())
     { }
 
     /**
@@ -488,9 +534,9 @@ public:
      * @returns
      * true if there are further tokens, false otherwise
      */
-    bool hasMoreTokens() const
+    bool hasMoreTokens() const override
 	{
-        return _tokIter != _tok.end();
+		return !_tokIter.isExhausted();
     }
 
     /**
@@ -504,23 +550,24 @@ public:
      * @pre
      * hasMoreTokens() must be true, otherwise an exception will be thrown.
      */
-    std::string nextToken() {
-        if (hasMoreTokens())
-            return *(_tokIter++);
-        else
-            throw ParseException("DefTokeniser: no more tokens");
+    std::string nextToken() override
+	{
+		if (hasMoreTokens())
+		{
+			return *(_tokIter++);
+		}
+        
+		throw ParseException("SingleCodeFileTokeniser: no more tokens");
     }
 
-	std::string peek() const
+	std::string peek() const override
 	{
 		if (hasMoreTokens())
 		{
             return *_tokIter;
 		}
-        else
-		{
-			throw ParseException("DefTokeniser: no more tokens");
-		}
+        
+		throw ParseException("SingleCodeFileTokeniser: no more tokens");
 	}
 };
 
@@ -564,9 +611,9 @@ private:
 
 	typedef std::list<std::string> StringList;
 
-	// A map associating #define names with a one or more sub-tokens
-	typedef std::map<std::string, StringList> DefinitionMap;
-	DefinitionMap _definitions;
+	// A map associating names to #define'd macros
+	typedef std::map<std::string, Macro> Macros;
+	Macros _macros;
 
 	// A small local buffer which is needed to properly resolve #define statements
 	// which could consist of several tokens themselves
@@ -645,7 +692,7 @@ private:
 			// Don't treat #strNNNN as preprocessor tokens
 			if (!token.empty() &&
 				token[0] == '#' &&
-				!boost::algorithm::starts_with(token, "#str"))
+				!string::starts_with(token, "#str"))
 			{
 				// A pre-processor token is ahead
 				handlePreprocessorToken(token);
@@ -654,40 +701,118 @@ private:
 
 			_tokenBuffer.push_front(token);
 
-			// Found a non-preprocessor token
+			// Found a non-preprocessor token,
+			// check if this is matching a preprocessor definition
+			Macros::const_iterator found = _macros.find(_tokenBuffer.front());
 
-			while (!_tokenBuffer.empty())
+			if (found != _macros.end())
 			{
-				// Exit if the first buffer token is processed
-				if (!processFirstBufferToken())
+				// Expand this macro, new tokens are acquired from the currently active tokeniser
+				StringList expanded = expandMacro(found->second, [this]() { return (*_curNode)->tokeniser.nextToken(); });
+
+				if (!expanded.empty())
 				{
-					return;
+					// Replace the token in the buffer with the expanded string
+					_tokenBuffer.pop_front();
+					_tokenBuffer.insert(_tokenBuffer.begin(), expanded.begin(), expanded.end());
 				}
 			}
+
+			return; // got a token
 		}
 	}
 
-	// Checks if the first token in the buffer is to be replaced by some #define
-	// Returns true if the token got processed (the buffer is changed then)
-	bool processFirstBufferToken()
+	// Expands the given macro, returns the beginning and end of a range
+	StringList expandMacro(const Macro& macro, const std::function<std::string()>& nextTokenFunc)
 	{
-		if (_tokenBuffer.empty()) return false;
+		StringList argumentValues;
 
-		// Check if this is matching a preprocessor definition
-		DefinitionMap::const_iterator found = _definitions.find(_tokenBuffer.front());
-
-		if (found != _definitions.end())
+		// Acquire the macro argument values if applicable
+		if (!macro.arguments.empty())
 		{
-			// Remove the token
-			_tokenBuffer.pop_front();
+			// Assert an opening parenthesis
+			if (nextTokenFunc() != "(")
+			{
+				throw ParseException(fmt::format("Error expanding macro {0}, expected '('", macro.name));
+			}
 
-			// Replace the token by the #defined contents
-			_tokenBuffer.insert(_tokenBuffer.begin(), found->second.begin(), found->second.end());
+			for (StringList::const_iterator i = macro.arguments.begin(); i != macro.arguments.end(); ++i)
+			{
+				std::string argumentToken = nextTokenFunc();
+				std::string argumentValue = "";
 
-			return true;
+				while (argumentToken != "," && argumentToken != ")")
+				{
+					argumentValue += argumentToken;
+					argumentValue.append(" ");
+
+					argumentToken = nextTokenFunc();
+				}
+
+				argumentValues.push_back(argumentValue);
+			}
 		}
 
-		return false;
+		// Allocate a new list for the expanded tokens
+		StringList expandedTokens;
+		
+		// Insert the macro contents into the buffer, expanding sub-macros along the way
+		for (StringList::const_iterator t = macro.tokens.begin(); t != macro.tokens.end(); ++t)
+		{
+			// Replace macro variable with its value
+			std::string token = getMacroToken(*t, macro, argumentValues);
+
+			// check if this is matching a preprocessor definition
+			Macros::const_iterator found = _macros.find(token);
+
+			if (found == _macros.end())
+			{
+				// Not a macro
+				expandedTokens.push_back(token);
+				continue;
+			}
+
+			// Enter recursion to expand this sub-macro, new tokens are acquired from the current iterator t
+			StringList subMacro = expandMacro(found->second, [&]() 
+			{ 
+				if (t == macro.tokens.end())
+				{
+					throw ParseException(fmt::format("Running out of tokens expanding sub-macro {0}", token));
+				}
+
+				// Advance iterator and dereference
+				// When delivering tokens to sub-macro expansions we still need to replace any argument values
+				return getMacroToken(*(++t), macro, argumentValues);
+			});
+
+			if (!subMacro.empty())
+			{
+				// Insert the expanded macro contents, don't insert *t itself
+				expandedTokens.insert(expandedTokens.end(), subMacro.begin(), subMacro.end());
+			}
+			else
+			{
+				rWarning() << "Macro expansion yields empty token list: " << *t <<
+					" in " << (*_curNode)->archive->getName() << std::endl;
+			}
+		}
+
+		return expandedTokens;
+	}
+
+	std::string getMacroToken(std::string token, const Macro& macro, const StringList& argumentValues)
+	{
+		// Check if the current token is referring to a macro argument and replace it with its value
+		for (StringList::const_iterator arg = macro.arguments.begin(), val = argumentValues.begin();
+			arg != macro.arguments.end() && val != argumentValues.end(); ++arg, ++val)
+		{
+			if (token == *arg)
+			{
+				return *val;
+			}
+		}
+
+		return token; // leave token unchanged
 	}
 
 	void handlePreprocessorToken(const std::string& token)
@@ -725,72 +850,31 @@ private:
 					<< includeFile << " in " << (*_curNode)->archive->getName() << std::endl;
 			}
 		}
-		else if (boost::algorithm::starts_with(token, "#define"))
+		else if (string::starts_with(token, "#define"))
 		{
-			std::string defineToken = token;
-
-			if (defineToken.length() <= 7)
-			{
-				rWarning() << "Invalid #define statement: "
-					<< " in " << (*_curNode)->archive->getName() << std::endl;
-				return;
-			}
-
-			// Replace tabs with spaces
-			std::replace(defineToken.begin(), defineToken.end(), '\t', ' ');
-
-			// Cut off the "#define " (including space)
-			std::string key = defineToken.substr(8);
-
-			std::size_t firstSpace = key.find(' ');
-
-			// Extract the value (can be empty) and trim it (everything after the space)
-			std::string value = (firstSpace == std::string::npos) ? "" : key.substr(firstSpace + 1);
-			boost::algorithm::trim(value);
-
-			key = key.substr(0, firstSpace);
-
-			std::pair<DefinitionMap::iterator, bool> result = _definitions.insert(
-				DefinitionMap::value_type(key, StringList())
-			);
-
-			if (!result.second)
-			{
-				rWarning() << "Redefinition of " << key
-					<< " in " << (*_curNode)->archive->getName() << std::endl;
-
-				result.first->second.clear();
-			}
-
-			// Instantiate a local stringtokeniser to split up the #defined string
-			BasicStringTokeniser tokeniser(value, _delims);
-
-			while (tokeniser.hasMoreTokens())
-			{
-				result.first->second.push_back(tokeniser.nextToken());
-			}
+			parseMacro(token);
 		}
 		else if (token == "#undef")
 		{
 			std::string key = (*_curNode)->tokeniser.nextToken();
-			_definitions.erase(key);
+			_macros.erase(key);
 		}
 		else if (token == "#ifdef")
 		{
 			std::string key = (*_curNode)->tokeniser.nextToken();
-			DefinitionMap::const_iterator found = _definitions.find(key);
+			Macros::const_iterator found = _macros.find(key);
 
-			if (found == _definitions.end())
+			if (found == _macros.end())
 			{
 				skipInactivePreprocessorBlock();
 			}
 		}
 		else if (token == "#ifndef")
 		{
-			DefinitionMap::const_iterator found = _definitions.find(
+			Macros::const_iterator found = _macros.find(
 				(*_curNode)->tokeniser.nextToken());
 
-			if (found != _definitions.end())
+			if (found != _macros.end())
 			{
 				skipInactivePreprocessorBlock();
 			}
@@ -806,6 +890,88 @@ private:
 		}
 	}
 
+	void parseMacro(const std::string& token)
+	{
+		std::string defineToken = token;
+
+		if (defineToken.length() <= 7)
+		{
+			rWarning() << "Invalid #define statement in " << (*_curNode)->archive->getName() << std::endl;
+			return;
+		}
+
+		// Replace tabs with spaces
+		std::replace(defineToken.begin(), defineToken.end(), '\t', ' ');
+
+		// Cut off the "#define " (including space)
+		defineToken = defineToken.substr(8);
+
+		// Parse the entire macro
+		std::istringstream macroStream(defineToken);
+		SingleCodeFileTokeniser macroParser(macroStream);
+
+		std::string name = macroParser.nextToken();
+
+		bool paramsStarted = false;
+
+		if (string::ends_with(name, "("))
+		{
+			string::trim_right(name, "(");
+			paramsStarted = true;
+		}
+
+		std::pair<Macros::iterator, bool> result = _macros.insert(
+			Macros::value_type(name, Macro(name))
+		);
+
+		if (!result.second)
+		{
+			rWarning() << "Redefinition of " << name << " in " << (*_curNode)->archive->getName() << std::endl;
+			result.first->second = Macro(name);
+		}
+
+		Macro& macro = result.first->second;
+
+		while (macroParser.hasMoreTokens())
+		{
+			std::string macroToken = macroParser.nextToken();
+
+			// An opening parenthesis might be an argument list, but
+			// only if we're still at the beginning of the macro
+			if (macroToken == "(" && !paramsStarted && macro.tokens.empty())
+			{	
+				paramsStarted = true;
+			}
+			else if (macroToken == ")" && paramsStarted)
+			{
+				paramsStarted = false;
+			}
+			else if (macroToken == ",")
+			{
+				if (paramsStarted)
+				{
+					continue;
+				}
+
+				// Treat the comma as part of the macro value
+				macro.tokens.push_back(macroToken);
+			}
+			else 
+			{
+				if (paramsStarted)
+				{
+					// Token is an argument
+					macro.arguments.push_back(macroToken);
+				}
+				else
+				{
+					// Ordinary macro value
+					macro.tokens.push_back(macroToken);
+				}
+			}
+		}
+	}
+
 	void skipInactivePreprocessorBlock()
 	{
 		// Not defined, skip everything until matching #endif
@@ -837,5 +1003,3 @@ private:
 };
 
 } // namespace parser
-
-#endif /* _CODE_TOKENISER_H_ */
