@@ -3,7 +3,7 @@
 #include "ifilter.h"
 #include "i18n.h"
 #include "igl.h"
-#include "icamera.h"
+#include "icameraview.h"
 #include "iscenegraphfactory.h"
 #include "irendersystemfactory.h"
 #include "iuimanager.h"
@@ -109,23 +109,23 @@ void RenderPreview::setupToolbars(bool enableAnimation)
     filterToolbar->Realize();
 
     // Get notified of filter changes
-    GlobalFilterSystem().filtersChangedSignal().connect(
-        sigc::mem_fun(this, &RenderPreview::filtersChanged)
-        );
+    GlobalFilterSystem().filterConfigChangedSignal().connect(
+        sigc::mem_fun(this, &RenderPreview::onFilterConfigChanged)
+    );
 
     wxToolBar* renderToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewRenderModeToolbar");
 
-    renderToolbar->Connect(getToolBarToolByLabel(renderToolbar, "texturedModeButton")->GetId(),
-                           wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onRenderModeChanged), NULL, this);
-    renderToolbar->Connect(getToolBarToolByLabel(renderToolbar, "lightingModeButton")->GetId(),
-                           wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onRenderModeChanged), NULL, this);
+    renderToolbar->Bind(wxEVT_TOOL, &RenderPreview::onRenderModeChanged, this, 
+        getToolBarToolByLabel(renderToolbar, "texturedModeButton")->GetId());
+    renderToolbar->Bind(wxEVT_TOOL, &RenderPreview::onRenderModeChanged, this,
+        getToolBarToolByLabel(renderToolbar, "lightingModeButton")->GetId());
 
     updateActiveRenderModeButton();
 
 	wxToolBar* utilToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewUtilToolbar");
 
-	utilToolbar->Connect(getToolBarToolByLabel(utilToolbar, "gridButton")->GetId(),
-		wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onGridButtonClick), NULL, this);
+	utilToolbar->Bind(wxEVT_TOOL, &RenderPreview::onGridButtonClick, this,
+        getToolBarToolByLabel(utilToolbar, "gridButton")->GetId());
 
 	utilToolbar->ToggleTool(getToolBarToolByLabel(utilToolbar, "gridButton")->GetId(), _renderGrid);
 }
@@ -166,7 +166,7 @@ void RenderPreview::updateActiveRenderModeButton()
     }
 }
 
-void RenderPreview::filtersChanged()
+void RenderPreview::onFilterConfigChanged()
 {
     if (!getScene()->root()) return;
 
@@ -213,7 +213,7 @@ void RenderPreview::initialisePreview()
     glLightfv(GL_LIGHT1, GL_DIFFUSE, l1Dif);
     glLightfv(GL_LIGHT1, GL_POSITION, l1Pos);
 
-    if (GlobalOpenGL().shaderProgramsAvailable())
+    if (_renderSystem->shaderProgramsAvailable())
     {
         setLightingModeEnabled(false);
     }
@@ -330,7 +330,7 @@ Matrix4 RenderPreview::calculateModelViewMatrix()
 
     // Rotate the view like a human would turn their head. Due to the radiant2openGL transform
     // the axes are used differently. Pitch is rotating around y instead of x, for example.
-    Vector3 radiant_eulerXYZ(0, _viewAngles[ui::CAMERA_PITCH], -_viewAngles[ui::CAMERA_YAW]);
+    Vector3 radiant_eulerXYZ(0, _viewAngles[camera::CAMERA_PITCH], -_viewAngles[camera::CAMERA_YAW]);
     modelview.rotateByEulerXYZDegrees(radiant_eulerXYZ);
 
     // As last step apply the radiant2openGL transform which rotates first around z, then around y
@@ -419,9 +419,9 @@ RenderStateFlags RenderPreview::getRenderFlagsWireframe()
            RENDER_PROGRAM;
 }
 
-void RenderPreview::drawPreview()
+bool RenderPreview::drawPreview()
 {
-    if (_renderingInProgress) return; // avoid double-entering this method
+    if (_renderingInProgress) return false; // avoid double-entering this method
 
     if (!_initialised)
     {
@@ -451,7 +451,7 @@ void RenderPreview::drawPreview()
 	{
 		// a return value of false means to cancel rendering
 		drawTime();
-		return;
+		return true; // swap buffers
 	}
 
     // Set up the camera
@@ -489,6 +489,8 @@ void RenderPreview::drawPreview()
 
     // Draw the render time
     drawTime();
+
+    return true;
 }
 
 void RenderPreview::renderWireFrame()
@@ -568,18 +570,18 @@ void RenderPreview::onGLMotionDelta(int x, int y, unsigned int mouseState)
     const float dtime = 0.1f;
     const float angleSpeed = 3; // camerasettings::anglespeed
 
-    _viewAngles[ui::CAMERA_PITCH] += y * dtime * angleSpeed;
-    _viewAngles[ui::CAMERA_YAW] += x * dtime * angleSpeed;
+    _viewAngles[camera::CAMERA_PITCH] += y * dtime * angleSpeed;
+    _viewAngles[camera::CAMERA_YAW] += x * dtime * angleSpeed;
 
-    if (_viewAngles[ui::CAMERA_PITCH] > 90)
-        _viewAngles[ui::CAMERA_PITCH] = 90;
-    else if (_viewAngles[ui::CAMERA_PITCH] < -90)
-        _viewAngles[ui::CAMERA_PITCH] = -90;
+    if (_viewAngles[camera::CAMERA_PITCH] > 90)
+        _viewAngles[camera::CAMERA_PITCH] = 90;
+    else if (_viewAngles[camera::CAMERA_PITCH] < -90)
+        _viewAngles[camera::CAMERA_PITCH] = -90;
 
-    if (_viewAngles[ui::CAMERA_YAW] >= 360)
-        _viewAngles[ui::CAMERA_YAW] -= 360;
-    else if (_viewAngles[ui::CAMERA_YAW] <= 0)
-        _viewAngles[ui::CAMERA_YAW] += 360;
+    if (_viewAngles[camera::CAMERA_YAW] >= 360)
+        _viewAngles[camera::CAMERA_YAW] -= 360;
+    else if (_viewAngles[camera::CAMERA_YAW] <= 0)
+        _viewAngles[camera::CAMERA_YAW] += 360;
 
     updateModelViewMatrix();
 
