@@ -277,7 +277,7 @@ void ConversationDialog::onAddEntity(wxCommandEvent& ev)
     if (eclass)
     {
         // Construct a Node of this entity type
-        IEntityNodePtr node(GlobalEntityCreator().createEntity(eclass));
+        IEntityNodePtr node(GlobalEntityModule().createEntity(eclass));
 
         // Create a random offset
 		node->getEntity().setKeyValue(
