@@ -20,10 +20,12 @@
 namespace ui
 {
 
-ResponseEditor::ResponseEditor(wxWindow* parent, StimTypes& stimTypes) :
-	ClassEditor(parent, stimTypes)
+ResponseEditor::ResponseEditor(wxWindow* mainPanel, StimTypes& stimTypes) :
+	ClassEditor(mainPanel, stimTypes),
+	_mainPanel(mainPanel)
 {
-	populatePage(this);
+	setupPage();
+
 	createContextMenu();
 	update();
 }
@@ -59,15 +61,16 @@ void ResponseEditor::update()
 {
 	_updatesDisabled = true;
 
-	wxPanel* mainPanel = findNamedObject<wxPanel>(this, "ResponseEditorMainPanel");
+	wxPanel* mainPanel = findNamedObject<wxPanel>(_mainPanel, "SREditorResponsePanel");
+	auto removeButton = findNamedObject<wxButton>(_mainPanel, "RemoveResponseButton");
 
-	int id = getIdFromSelection();
+	int index = getIndexFromSelection();
 
-	if (id > 0 && _entity != NULL)
+	if (index > 0 && _entity != NULL)
 	{
 		mainPanel->Enable(true);
 
-		StimResponse& sr = _entity->get(id);
+		StimResponse& sr = _entity->get(index);
 
 		// Get the iter into the liststore pointing at the correct STIM_YYYY type
 		std::string typeToFind = sr.get("type");
@@ -105,6 +108,7 @@ void ResponseEditor::update()
 
 		// Update the delete context menu item
 		_contextMenu.remove->Enable(!sr.inherited());
+		removeButton->Enable(!sr.inherited());
 
 		// If there is anything selected, the duplicate item is always active
 		_contextMenu.duplicate->Enable(true);
@@ -133,47 +137,49 @@ void ResponseEditor::update()
 		_contextMenu.disable->Enable(false);
 		_contextMenu.remove->Enable(false);
 		_contextMenu.duplicate->Enable(false);
+
+		removeButton->Enable(false);
 	}
 
 	_updatesDisabled = false;
 }
 
-void ResponseEditor::populatePage(wxWindow* parent)
+void ResponseEditor::setupPage()
 {
-	wxPanel* editingPanel = loadNamedPanel(parent, "ResponseEditorMainPanel");
-	packEditingPane(editingPanel);
+	wxPanel* listPanel = findNamedObject<wxPanel>(_mainPanel, "SREditorResponseList");
+	createListView(listPanel);
 
 #ifdef USE_BMP_COMBO_BOX
 	// Response property section
-	_type = findNamedObject<wxBitmapComboBox>(this, "ResponseEditorTypeCombo");
+	_type = findNamedObject<wxBitmapComboBox>(_mainPanel, "ResponseEditorTypeCombo");
 #else
-	// Response property section
-	wxControl* typeBox = findNamedObject<wxControl>(this, "ResponseEditorTypeCombo");
+	{
+		// Response property section
+		wxControl* typeBox = findNamedObject<wxControl>(_mainPanel, "ResponseEditorTypeCombo");
+	
+		// Replace the bitmap combo with an ordinary one
+		wxComboBox* combo = new wxComboBox(typeBox->GetParent(), wxID_ANY);
+		typeBox->GetContainingSizer()->Add(combo, 1, wxEXPAND);
+		typeBox->Destroy();
 
-	// Replace the bitmap combo with an ordinary one
-	wxComboBox* combo = new wxComboBox(typeBox->GetParent(), wxID_ANY);
-	typeBox->GetContainingSizer()->Add(combo, 1, wxEXPAND);
-	typeBox->Destroy();
-
-	_type = combo;
-	_type->SetName("ResponseEditorTypeCombo");
+		_type = combo;
+		_type->SetName("ResponseEditorTypeCombo");
+	}
 #endif
 
-	_stimTypes.populateComboBox(_type);
-
 	_type->Connect(wxEVT_COMBOBOX, wxCommandEventHandler(ResponseEditor::onStimTypeSelect), NULL, this); 
 
 	// Active
-	_propertyWidgets.active = findNamedObject<wxCheckBox>(this, "ResponseEditorActive");
+	_propertyWidgets.active = findNamedObject<wxCheckBox>(_mainPanel, "ResponseEditorActive");
 
 	// Random Effects Toggle
-	_propertyWidgets.randomEffectsToggle = findNamedObject<wxCheckBox>(this, "ResponseEditorRandomFX");
-	_propertyWidgets.randomEffectsEntry = findNamedObject<wxTextCtrl>(this, "ResponseEditorRandomFXValue");
+	_propertyWidgets.randomEffectsToggle = findNamedObject<wxCheckBox>(_mainPanel, "ResponseEditorRandomFX");
+	_propertyWidgets.randomEffectsEntry = findNamedObject<wxTextCtrl>(_mainPanel, "ResponseEditorRandomFXValue");
 
 	// Chance variable
-	_propertyWidgets.chanceToggle = findNamedObject<wxCheckBox>(this, "ResponseEditorChance");
+	_propertyWidgets.chanceToggle = findNamedObject<wxCheckBox>(_mainPanel, "ResponseEditorChance");
 
-	wxPanel* chancePanel = findNamedObject<wxPanel>(this, "ResponseEditorChanceValuePanel");
+	wxPanel* chancePanel = findNamedObject<wxPanel>(_mainPanel, "ResponseEditorChanceValuePanel");
 
 	_propertyWidgets.chanceEntry = new wxSpinCtrlDouble(chancePanel, wxID_ANY);
 	_propertyWidgets.chanceEntry->SetRange(0.0, 1.0);
@@ -191,20 +197,51 @@ void ResponseEditor::populatePage(wxWindow* parent)
 
 	connectSpinButton(_propertyWidgets.chanceEntry, "chance");
 
-	makeLabelBold(this, "ResponseEditorFXLabel");
+	makeLabelBold(_mainPanel, "ResponseEditorFXLabel");
 
 	createEffectWidgets();
 
-	editingPanel->Layout();
-	editingPanel->Fit();
-	Layout();
-	Fit();
+#ifdef USE_BMP_COMBO_BOX
+	_addType = findNamedObject<wxBitmapComboBox>(_mainPanel, "ResponseTypeComboBox");
+#else
+	{
+		// Type selector box
+		wxControl* addTypeBox = findNamedObject<wxControl>(_mainPanel, "ResponseTypeComboBox");
+
+		// Replace the bitmap combo with an ordinary one
+		wxComboBox* newTypeCombo = new wxComboBox(addTypeBox->GetParent(), wxID_ANY);
+		addTypeBox->GetContainingSizer()->Prepend(newTypeCombo, 1, wxEXPAND | wxRIGHT, 6);
+		addTypeBox->Destroy();
+
+		_addType = newTypeCombo;
+		_addType->SetName("ResponseTypeComboBox");
+	}
+#endif
+	_addType->Bind(wxEVT_COMBOBOX, std::bind(&ResponseEditor::onAddTypeSelect, this, std::placeholders::_1));
+
+	auto addButton = findNamedObject<wxButton>(_mainPanel, "AddResponseButton");
+	auto removeButton = findNamedObject<wxButton>(_mainPanel, "RemoveResponseButton");
+
+	addButton->Bind(wxEVT_BUTTON, std::bind(&ResponseEditor::onAddSR, this, std::placeholders::_1));
+	removeButton->Bind(wxEVT_BUTTON, std::bind(&ResponseEditor::onRemoveSR, this, std::placeholders::_1));
+	
+	reloadStimTypes();
+}
+
+void ResponseEditor::reloadStimTypes()
+{
+	if (_stimTypes.getStimMap().empty())
+	{
+		_stimTypes.reload();
+	}
+
+	_stimTypes.populateComboBox(_addType);
+	_stimTypes.populateComboBox(_type);
 }
 
-// Create the response effect list widgets
 void ResponseEditor::createEffectWidgets()
 {
-	wxPanel* effectsPanel = findNamedObject<wxPanel>(this, "ResponseEditorFXPanel");
+	wxPanel* effectsPanel = findNamedObject<wxPanel>(_mainPanel, "ResponseEditorFXPanel");
 
 	wxutil::TreeModel::Ptr dummyModel(
         new wxutil::TreeModel(StimResponse::getColumns(), true)
@@ -268,13 +305,13 @@ void ResponseEditor::checkBoxToggled(wxCheckBox* toggleButton)
 
 void ResponseEditor::addEffect()
 {
-	if (_entity == NULL) return;
+	if (_entity == nullptr) return;
 
-	int id = getIdFromSelection();
+	int index = getIndexFromSelection();
 
-	if (id > 0)
+	if (index > 0)
 	{
-		StimResponse& sr = _entity->get(id);
+		StimResponse& sr = _entity->get(index);
 		int effectIndex = getEffectIdFromSelection();
 
 		// Make sure we have a response
@@ -288,13 +325,13 @@ void ResponseEditor::addEffect()
 
 void ResponseEditor::removeEffect()
 {
-	if (_entity == NULL) return;
+	if (_entity == nullptr) return;
 
-	int id = getIdFromSelection();
+	int index = getIndexFromSelection();
 
-	if (id > 0)
+	if (index > 0)
 	{
-		StimResponse& sr = _entity->get(id);
+		StimResponse& sr = _entity->get(index);
 		int effectIndex = getEffectIdFromSelection();
 
 		// Make sure we have a response and anything selected
@@ -309,20 +346,20 @@ void ResponseEditor::removeEffect()
 
 void ResponseEditor::editEffect()
 {
-	if (_entity == NULL) return;
+	if (_entity == nullptr) return;
 
-	int id = getIdFromSelection();
+	int index = getIndexFromSelection();
 
-	if (id > 0)
+	if (index > 0)
 	{
-		StimResponse& sr = _entity->get(id);
+		StimResponse& sr = _entity->get(index);
 		int effectIndex = getEffectIdFromSelection();
 
 		// Make sure we have a response and anything selected
 		if (sr.get("class") == "R" && effectIndex > 0)
 		{
 			// Create a new effect editor (self-destructs)
-			EffectEditor* editor = new EffectEditor(this, sr, effectIndex, _stimTypes, *this);
+			EffectEditor* editor = new EffectEditor(_mainPanel, sr, effectIndex, _stimTypes, *this);
 
 			editor->ShowModal();
 			editor->Destroy();
@@ -334,11 +371,11 @@ void ResponseEditor::moveEffect(int direction)
 {
 	if (_entity == NULL) return;
 
-	int id = getIdFromSelection();
+	int index = getIndexFromSelection();
 
-	if (id > 0)
+	if (index > 0)
 	{
-		StimResponse& sr = _entity->get(id);
+		StimResponse& sr = _entity->get(index);
 		int effectIndex = getEffectIdFromSelection();
 
 		if (sr.get("class") == "R" && effectIndex > 0)
@@ -360,11 +397,11 @@ void ResponseEditor::updateEffectContextMenu()
 
 	bool anythingSelected = curEffectIndex >= 0;
 
-	int srId = getIdFromSelection();
+	int index = getIndexFromSelection();
 
-	if (srId > 0)
+	if (index > 0)
 	{
-		StimResponse& sr = _entity->get(srId);
+		StimResponse& sr = _entity->get(index);
 		highestEffectIndex = sr.highestEffectIndex();
 	}
 
@@ -476,10 +513,10 @@ void ResponseEditor::addSR()
 	if (_entity == NULL) return;
 
 	// Create a new StimResponse object
-	int id = _entity->add();
+	int index = _entity->add();
 
 	// Get a reference to the newly allocated object
-	StimResponse& sr = _entity->get(id);
+	StimResponse& sr = _entity->get(index);
 	sr.set("class", "R");
 
 	// Get the selected stim type name from the combo box
@@ -492,7 +529,7 @@ void ResponseEditor::addSR()
 	_entity->updateListStores();
 
 	// Select the newly created response
-	selectId(id);
+	selectIndex(index);
 }
 
 void ResponseEditor::onEffectItemContextMenu(wxDataViewEvent& ev)
