@@ -2,9 +2,11 @@
 
 #include "iscenegraph.h"
 #include "i18n.h"
-#include "iuimanager.h"
+#include "icolourscheme.h"
 #include "registry/registry.h"
-#include "Device.h"
+#include "selection/Device.h"
+#include "Rectangle.h"
+#include "selection/SelectionVolume.h"
 #include "igl.h"
 
 namespace ui
@@ -96,7 +98,7 @@ void DragSelectionMouseTool::renderOverlay()
     glBlendColor(0, 0, 0, 0.2f);
     glBlendFunc(GL_CONSTANT_ALPHA_EXT, GL_ONE_MINUS_CONSTANT_ALPHA_EXT);
 
-    Vector3 dragBoxColour = ColourSchemes().getColour("drag_selection");
+    Vector3 dragBoxColour = GlobalColourSchemeManager().getColour("drag_selection");
     glColor3dv(dragBoxColour);
     glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
@@ -130,14 +132,28 @@ void DragSelectionMouseTool::testSelect(MouseTool::Event& ev)
     // If the mouse pointer has moved more than <epsilon>, this is considered a drag operation
     if (fabs(delta.x()) > _epsilon.x() && fabs(delta.y()) > _epsilon.y())
     {
+        // Construct the selection test according to the area the user covered with his drag
+        render::View scissored(_view);
+        ConstructSelectionTest(scissored, selection::Rectangle::ConstructFromArea(_start, delta));
+
+        SelectionVolume volume(scissored);
+
         // Call the selectArea command that does the actual selecting
-        GlobalSelectionSystem().SelectArea(_view, _start, delta, SelectionSystem::eToggle, isFaceOperation);
+        GlobalSelectionSystem().selectArea(volume, SelectionSystem::eToggle, isFaceOperation);
     }
     else
     {
+        // Copy the view to create a scissored volume
+        render::View scissored(_view);
+        // Create a volume out of a small box with 2*epsilon edge length
+        ConstructSelectionTest(scissored, 
+            selection::Rectangle::ConstructFromPoint(ev.getDevicePosition(), _epsilon));
+
+        // Create a selection test using that volume
+        SelectionVolume volume(scissored);
+
         // Mouse has barely moved, call the point selection routine
-        GlobalSelectionSystem().SelectPoint(_view, ev.getDevicePosition(),
-                                            _epsilon, SelectionSystem::eToggle, isFaceOperation);
+        GlobalSelectionSystem().selectPoint(volume, SelectionSystem::eToggle, isFaceOperation);
     }
 
     // Reset the mouse position to zero, this mouse operation is finished so far
@@ -229,8 +245,15 @@ void CycleSelectionMouseTool::testSelect(MouseTool::Event& ev)
     // If we already replaced a selection, switch to cycle mode
     // eReplace should only be active during the first call without mouse movement
     SelectionSystem::EModifier modifier = _mouseMovedSinceLastSelect ? SelectionSystem::eReplace : SelectionSystem::eCycle;
+    
+    // Copy the view to create a scissored volume
+    render::View scissored(_view);
+    // Create a volume out of a small box with 2*epsilon edge length
+    ConstructSelectionTest(scissored, selection::Rectangle::ConstructFromPoint(curPos, _epsilon));
 
-    GlobalSelectionSystem().SelectPoint(_view, curPos, _epsilon, modifier, selectFacesOnly());
+    // Create a selection test using that volume
+    SelectionVolume volume(scissored);
+    GlobalSelectionSystem().selectPoint(volume, modifier, selectFacesOnly());
 
     // Remember this position
     _lastSelectPos = curPos;
