@@ -14,16 +14,14 @@
 #include "imainframe.h"
 #include "itextstream.h"
 
-#include "modulesystem/StaticModule.h"
+#include "module/StaticModule.h"
 #include "selectionlib.h"
+#include "scene/SelectionIndex.h"
 #include "scenelib.h"
 #include "wxutil/dialog/MessageBox.h"
 #include "wxutil/menu/IconTextMenuItem.h"
 #include "wxutil/TreeModel.h"
 #include "xmlutil/Document.h"
-#include "map/Map.h"
-#include "selection/algorithm/Entity.h"
-#include "selection/algorithm/General.h"
 
 #include <map>
 #include <string>
@@ -329,7 +327,7 @@ void EntityInspector::createContextMenu()
     );
 }
 
-void EntityInspector::onRadiantStartup()
+void EntityInspector::onMainFrameConstructed()
 {
     // Add entity inspector to the group dialog
     IGroupDialog::PagePtr page(new IGroupDialog::Page);
@@ -344,7 +342,7 @@ void EntityInspector::onRadiantStartup()
     GlobalGroupDialog().addPage(page);
 }
 
-void EntityInspector::onRadiantShutdown()
+void EntityInspector::onMainFrameShuttingDown()
 {
     _undoHandler.disconnect();
     _redoHandler.disconnect();
@@ -391,19 +389,18 @@ const StringSet& EntityInspector::getDependencies() const
     return _dependencies;
 }
 
-void EntityInspector::initialiseModule(const ApplicationContext& ctx)
+void EntityInspector::initialiseModule(const IApplicationContext& ctx)
 {
     construct();
 
-    GlobalRadiant().signal_radiantStarted().connect(
-        sigc::mem_fun(this, &EntityInspector::onRadiantStartup)
+    GlobalMainFrame().signal_MainFrameConstructed().connect(
+        sigc::mem_fun(this, &EntityInspector::onMainFrameConstructed)
     );
-    GlobalRadiant().signal_radiantShutdown().connect(
-        sigc::mem_fun(this, &EntityInspector::onRadiantShutdown)
+    GlobalMainFrame().signal_MainFrameShuttingDown().connect(
+        sigc::mem_fun(this, &EntityInspector::onMainFrameShuttingDown)
     );
 
     GlobalCommandSystem().addCommand("ToggleEntityInspector", toggle);
-    GlobalEventManager().addCommand("ToggleEntityInspector", "ToggleEntityInspector");
 }
 
 void EntityInspector::registerPropertyEditor(const std::string& key, const IPropertyEditorPtr& editor)
@@ -627,22 +624,13 @@ void EntityInspector::setPropertyFromEntries()
     _keyEntry->SetValue(key);
     _valEntry->SetValue(val);
 
-    UndoableCommand cmd("entitySetProperty");
-
     // Pass the call to the specialised routine
     applyKeyValueToSelection(key, val);
 }
 
 void EntityInspector::applyKeyValueToSelection(const std::string& key, const std::string& val)
 {
-    try
-    {
-        selection::algorithm::setEntityKeyvalue(key, val);
-    }
-    catch (std::runtime_error& ex)
-    {
-        wxutil::Messagebox::ShowError(ex.what());
-    }
+    GlobalCommandSystem().executeCommand("SetEntityKeyValue", key, val);
 }
 
 void EntityInspector::loadPropertyMap()
@@ -699,33 +687,41 @@ bool EntityInspector::_testAddKey()
 
 void EntityInspector::_onDeleteKey()
 {
+    wxDataViewItemArray selectedItems;
+    _keyValueTreeView->GetSelections(selectedItems);
+
+    if (selectedItems.Count() == 0) return;
+
     assert(!_selectedEntity.expired());
+    Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
 
-    std::string key = getSelectedKey();
+    std::unique_ptr<UndoableCommand> cmd;
 
-    if (!key.empty())
+    for (const wxDataViewItem& item : selectedItems)
     {
-        UndoableCommand cmd("deleteProperty");
-        
-        Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
+        wxutil::TreeModel::Row row(item, *_kvStore);
+
+        if (!isItemDeletable(row))
+        {
+            continue;
+        }
+
+        if (!cmd)
+        {
+            cmd.reset(new UndoableCommand("deleteProperty"));
+        }
+
+        auto iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+        auto key = iconAndName.GetText().ToStdString();
+
+        // Clear the key after copying
         selectedEntity->setKeyValue(key, "");
     }
 }
 
 bool EntityInspector::_testDeleteKey()
 {
-    if (_selectedEntity.expired()) return false;
-
-    auto selectedKey = getSelectedKey();
-
-    if (selectedKey.empty() || selectedKey == "classname")
-    {
-        return false;
-    }
-
-    // Make sure the Delete item is only available for explicit
-    // (non-inherited) properties
-    return !getListSelectionBool(_columns.isInherited);
+    return _testNonEmptyAndDeletableSelection();
 }
 
 void EntityInspector::_onCopyKey()
@@ -746,18 +742,8 @@ void EntityInspector::_onCopyKey()
         std::string key = iconAndName.GetText().ToStdString();
         std::string value = row[_columns.value];
 
-        _clipBoard.push_back(std::make_pair(key, value));
+        _clipBoard.emplace_back(key, value);
     }
-#if 0
-    std::string key = getSelectedKey();
-    std::string value = getListSelection(_columns.value);
-
-    if (!key.empty())
-    {
-        _clipBoard.key = key;
-        _clipBoard.value = value;
-    }
-#endif
 }
 
 bool EntityInspector::_testCopyKey()
@@ -776,33 +762,51 @@ void EntityInspector::_onCutKey()
     Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
 
     _clipBoard.clear();
+    std::unique_ptr<UndoableCommand> cmd;
 
     for (const wxDataViewItem& item : selectedItems)
     {
         wxutil::TreeModel::Row row(item, *_kvStore);
 
-        wxDataViewIconText iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+        if (!isItemDeletable(row))
+        {
+            continue;
+        }
 
-        std::string key = iconAndName.GetText().ToStdString();
-        std::string value = row[_columns.value];
+        if (!cmd)
+        {
+            cmd.reset(new UndoableCommand("cutProperty"));
+        }
 
-        _clipBoard.push_back(std::make_pair(key, value));
+        auto iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+        auto key = iconAndName.GetText().ToStdString();
+        auto value = row[_columns.value];
 
-        // We don't delete any inherited key values
-        if (row[_columns.isInherited].getBool()) continue;
-        
-        // Don't delete any classnames either
-        if (key == "classname") continue;
-
-        UndoableCommand cmd("cutProperty");
+        _clipBoard.emplace_back(key, value);
 
         // Clear the key after copying
         selectedEntity->setKeyValue(key, "");
     }
 }
 
-bool EntityInspector::_testCutKey()
+bool EntityInspector::isItemDeletable(const wxutil::TreeModel::Row& row)
 {
+    auto iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+    auto key = iconAndName.GetText().ToStdString();
+
+    // We don't delete any inherited key values
+    if (row[_columns.isInherited].getBool()) return false;
+
+    // Don't delete any classnames either
+    if (key == "classname") return false;
+
+    return true;
+}
+
+bool EntityInspector::_testNonEmptyAndDeletableSelection()
+{
+    if (_selectedEntity.expired()) return false;
+    
     wxDataViewItemArray selectedItems;
     _keyValueTreeView->GetSelections(selectedItems);
 
@@ -810,18 +814,21 @@ bool EntityInspector::_testCutKey()
     {
         wxutil::TreeModel::Row row(item, *_kvStore);
 
-        wxDataViewIconText iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
-        std::string key = iconAndName.GetText().ToStdString();
-
-        if (!row[_columns.isInherited].getBool() && key != "classname")
+        if (isItemDeletable(row))
         {
             return true; // we have at least one non-inherited value that is not "classname"
         }
     }
 
+    // Either all keys are inherited or classname, or nothing selected at all
     return false;
 }
 
+bool EntityInspector::_testCutKey()
+{
+    return _testNonEmptyAndDeletableSelection();
+}
+
 void EntityInspector::_onPasteKey()
 {
     // greebo: Instantiate a scoped object to make this operation undoable
@@ -1180,10 +1187,9 @@ void EntityInspector::getEntityFromSelectionSystem()
         changeSelectedEntity(selectedNode);
 
         // Just set the entity number
-        std::size_t ent(0), prim(0);
-        selection::algorithm::getSelectionIndex(ent, prim);
+        auto indices = scene::getNodeIndices(selectedNode);
 
-        _primitiveNumLabel->SetLabelText(fmt::format(_("Entity {0}"), ent));
+        _primitiveNumLabel->SetLabelText(fmt::format(_("Entity {0}"), indices.first));
     }
     else
     {
@@ -1191,10 +1197,17 @@ void EntityInspector::getEntityFromSelectionSystem()
         scene::INodePtr selectedNodeParent = selectedNode->getParent();
         changeSelectedEntity(selectedNodeParent);
         
-        std::size_t ent(0), prim(0);
-        selection::algorithm::getSelectionIndex(ent, prim);
+        try
+        {
+            auto indices = scene::getNodeIndices(selectedNode);
 
-        _primitiveNumLabel->SetLabelText(fmt::format(_("Entity {0}, Primitive {1}"), ent, prim));
+            _primitiveNumLabel->SetLabelText(fmt::format(_("Entity {0}, Primitive {1}"), indices.first, indices.second));
+        }
+        catch (const std::out_of_range& ex)
+        {
+            rWarning() << ex.what() << std::endl;
+            _primitiveNumLabel->SetLabelText("-");
+        }
     }
 }
 
