@@ -4,6 +4,7 @@
 #include "iradiant.h"
 #include "ieventmanager.h"
 #include "idialogmanager.h"
+#include "icolourscheme.h"
 #include "wxutil/dialog/Dialog.h"
 #include "wxutil/dialog/MessageBox.h"
 #include "wxutil/EntryAbortedException.h"
@@ -16,7 +17,6 @@
 #include <wx/sizer.h>
 #include <wx/panel.h>
 
-#include "layers/LayerManager.h"
 #include "LayerControlDialog.h"
 
 namespace ui
@@ -39,9 +39,14 @@ LayerControl::LayerControl(wxWindow* parent, int layerID) :
 	_toggle = new wxBitmapToggleButton(parent, wxID_ANY, 
 		wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + ICON_LAYER_VISIBLE));
 	_toggle->SetMaxSize(wxSize(30, -1));
-	_toggle->Connect(wxEVT_TOGGLEBUTTON, wxCommandEventHandler(LayerControl::onToggle), NULL, this);
+	_toggle->Bind(wxEVT_TOGGLEBUTTON, &LayerControl::onToggle, this);
 
-	Vector3 selColour = ColourSchemes().getColour("selected_brush");
+#ifdef __WXMSW__
+    // Eat the double click event to not receive additional onToggle callbacks
+    _toggle->Bind(wxEVT_LEFT_DCLICK, [](wxMouseEvent& ev) {});
+#endif
+
+	Vector3 selColour = GlobalColourSchemeManager().getColour("selected_brush");
 	_activeColour = wxColour(static_cast<unsigned char>(selColour[0] * 255),
 		static_cast<unsigned char>(selColour[1] * 255),
 		static_cast<unsigned char>(selColour[2] * 255));
