@@ -1,6 +1,7 @@
 #pragma once
 
 #include "imenu.h"
+#include "ilayer.h"
 #include <memory>
 #include "LayerContextMenu.h"
 #include "wxutil/menu/IconTextMenuItem.h"
@@ -37,6 +38,9 @@ public:
 	static void AddToLayer(int layerID);
 	static void MoveToLayer(int layerID);
 	static void RemoveFromLayer(int layerID);
+
+private:
+	static void DoWithMapLayerManager(const std::function<void(scene::ILayerManager&)>& func);
 };
 typedef std::shared_ptr<LayerOrthoContextMenuItem> LayerOrthoContextMenuItemPtr;
 
