@@ -1,7 +1,8 @@
 #pragma once
 
-#include "map/ModelBreakdown.h"
+#include "scene/ModelBreakdown.h"
 
+#include "wxutil/menu/PopupMenu.h"
 #include <wx/panel.h>
 #include "wxutil/TreeView.h"
 
@@ -13,7 +14,7 @@ class ModelInfoTab :
 {
 private:
 	// The helper class counting the models in the map
-	map::ModelBreakdown _modelBreakdown;
+	scene::ModelBreakdown _modelBreakdown;
 
 	// Treemodel definition
 	struct ListColumns :
@@ -38,6 +39,8 @@ private:
 	wxutil::TreeModel::Ptr _listStore;
 	wxutil::TreeView* _treeView;
 
+    wxutil::PopupMenuPtr _popupMenu;
+
 public:
 	// Constructor
 	ModelInfoTab(wxWindow* parent);
@@ -49,6 +52,10 @@ private:
 	// This is called to create the widgets
 	void populateTab();
 
+    void _onSelectItems(bool select);
+    bool _testSelectItems();
+    void _onContextMenu(wxDataViewEvent& ev);
+
 }; // class ModelInfoTab
 
 } // namespace ui
