@@ -32,8 +32,8 @@ class TexturePreviewCombo;
  * into the texture window or applying directly to map geometry.
  */
 class MediaBrowser : 
-	public IMediaBrowser,
-	public wxEvtHandler
+	public wxEvtHandler,
+	public IMediaBrowser
 {
 public:
 	struct TreeColumns :
@@ -97,6 +97,9 @@ private:
 
 	sigc::connection _materialDefsLoaded;
 	sigc::connection _materialDefsUnloaded;
+	sigc::connection _shaderClipboardConn;
+
+	bool _blockShaderClipboardUpdates;
 
 private:
 	void construct();
@@ -154,7 +157,7 @@ public:
 
 	const std::string& getName() const override;
 	const StringSet& getDependencies() const override;
-	void initialiseModule(const ApplicationContext& ctx) override;
+	void initialiseModule(const IApplicationContext& ctx) override;
 	void shutdownModule() override;
 
 private:
@@ -162,8 +165,7 @@ private:
 	void onMaterialDefsUnloaded();
 	void onMaterialDefsLoaded();
 
-	// Radiant Event Listener
-	void onRadiantStartup();
+	void onMainFrameConstructed();
 
 	/**
 	* greebo: Command target for toggling the mediabrowser tab in the groupdialog.
@@ -171,6 +173,8 @@ private:
 	void togglePage(const cmd::ArgumentList& args);
 
 	void setupTreeViewAndFilter();
+
+	void onShaderClipboardSourceChanged();
 };
 
 }
