@@ -8,30 +8,30 @@
 #include "iresourcechooser.h"
 #include "idialogmanager.h"
 #include "entitylib.h" // EntityFindByClassnameWalker
+#include "selectionlib.h"
 #include "ientity.h" // Node_getEntity()
 #include "iregistry.h"
 #include "iuimanager.h"
 #include "imainframe.h"
-#include "map/Map.h"
-#include "modulesystem/ModuleRegistry.h"
 
 #include "wxutil/dialog/MessageBox.h"
 #include "wxutil/menu/IconTextMenuItem.h"
 #include "wxutil/menu/CommandMenuItem.h"
+#include "wxutil/EntityClassChooser.h"
 
-#include "selection/algorithm/Group.h"
-#include "selection/algorithm/ModelFinder.h"
-#include "selection/algorithm/Entity.h"
 #include "ui/modelselector/ModelSelector.h"
-#include "ui/entitychooser/EntityClassChooser.h"
+#include "ui/prefabselector/PrefabSelector.h"
 
 #include "string/convert.h"
+#include "scene/GroupNodeChecker.h"
+#include "scene/ModelFinder.h"
 #include "math/AABB.h"
 
 #include <wx/window.h>
 #include <wx/menu.h>
 
-#include "modulesystem/StaticModule.h"
+#include "module/StaticModule.h"
+#include "command/ExecutionFailure.h"
 
 namespace ui
 {
@@ -82,7 +82,8 @@ module::StaticModule<OrthoContextMenu> orthoContextMenuModule;
 
 OrthoContextMenu& OrthoContextMenu::Instance()
 {
-    return *orthoContextMenuModule.getModule();
+    return *std::static_pointer_cast<OrthoContextMenu>(
+        module::GlobalModuleRegistry().getModule("OrthoContextMenu"));
 }
 
 // Constructor
@@ -144,14 +145,14 @@ void OrthoContextMenu::analyseSelection()
     if (_selectionInfo.onlyEntitiesSelected)
     {
         // Check for group nodes
-        selection::algorithm::GroupNodeChecker walker;
+        scene::GroupNodeChecker walker;
         GlobalSelectionSystem().foreachSelected(walker);
 
         _selectionInfo.onlyGroupsSelected = walker.onlyGroupsAreSelected();
         _selectionInfo.singleGroupSelected = walker.selectedGroupCount() == 1 && !Node_isWorldspawn(walker.getFirstSelectedGroupNode());
 
         // Create a ModelFinder and check whether only models were selected
-        selection::algorithm::ModelFinder visitor;
+        scene::ModelFinder visitor;
         GlobalSelectionSystem().foreachSelected(visitor);
 
         // enable the "Add MonsterClip" entry only if at least one model is selected
@@ -221,7 +222,7 @@ bool OrthoContextMenu::checkMergeEntities()
 
 bool OrthoContextMenu::checkReparentPrimitives()
 {
-    return selection::algorithm::curSelectionIsSuitableForReparent();
+    return selection::curSelectionIsSuitableForReparent();
 }
 
 bool OrthoContextMenu::checkRevertToWorldspawnPartial()
@@ -260,7 +261,7 @@ void OrthoContextMenu::addEntity()
     UndoableCommand command("createEntity");
 
     // Display the chooser to select an entity classname
-    std::string cName = EntityClassChooser::chooseEntityClass();
+    std::string cName = wxutil::EntityClassChooser::chooseEntityClass();
 
     if (!cName.empty()) 
     {
@@ -268,9 +269,9 @@ void OrthoContextMenu::addEntity()
         // wrong number of brushes is selected.
         try
 		{
-            selection::algorithm::createEntityFromSelection(cName, _lastPoint);
+            GlobalEntityModule().createEntityFromSelection(cName, _lastPoint);
         }
-        catch (selection::algorithm::EntityCreationException& e)
+        catch (cmd::ExecutionFailure& e)
 		{
             wxutil::Messagebox::ShowError(e.what());
         }
@@ -284,15 +285,16 @@ void OrthoContextMenu::addPlayerStart()
     try
     {
         // Create the player start entity
-        scene::INodePtr playerStartNode = selection::algorithm::createEntityFromSelection(
+        auto playerStartNode = GlobalEntityModule().createEntityFromSelection(
             PLAYERSTART_CLASSNAME, _lastPoint
         );
-        Entity* playerStart = Node_getEntity(playerStartNode);
+        auto& playerStart = playerStartNode->getEntity();
 
         // Set a default angle
-        playerStart->setKeyValue(ANGLE_KEY_NAME, DEFAULT_ANGLE);
+        playerStart.setKeyValue(ANGLE_KEY_NAME, DEFAULT_ANGLE);
     }
-    catch (selection::algorithm::EntityCreationException& e) {
+    catch (cmd::ExecutionFailure& e)
+    {
         wxutil::Messagebox::ShowError(e.what());
     }
 }
@@ -306,7 +308,7 @@ void OrthoContextMenu::callbackMovePlayerStart()
 
     Entity* playerStart = walker.getEntity();
 
-    if (playerStart != NULL)
+    if (playerStart != nullptr)
     {
         playerStart->setKeyValue("origin", string::to_string(_lastPoint));
     }
@@ -318,18 +320,24 @@ void OrthoContextMenu::callbackAddLight()
 
     try 
 	{
-        selection::algorithm::createEntityFromSelection(LIGHT_CLASSNAME, _lastPoint);
+        GlobalEntityModule().createEntityFromSelection(LIGHT_CLASSNAME, _lastPoint);
     }
-    catch (selection::algorithm::EntityCreationException&)
+    catch (cmd::ExecutionFailure& e)
 	{
-        wxutil::Messagebox::ShowError(_("Unable to create light, classname not found."));
+        wxutil::Messagebox::ShowError(fmt::format(_("Unable to create light: {0}"), e.what()));
     }
 }
 
 void OrthoContextMenu::callbackAddPrefab()
 {
-    // Pass the call to the map algorithm and give the lastPoint coordinate as argument
-    GlobalMap().loadPrefabAt(_lastPoint);
+    auto result = PrefabSelector::ChoosePrefab();
+
+    if (!result.prefabPath.empty())
+    {
+        // Pass the call to the map algorithm and give the lastPoint coordinate as argument
+        GlobalCommandSystem().executeCommand(LOAD_PREFAB_AT_CMD, 
+            result.prefabPath, _lastPoint, result.insertAsGroup);
+    }
 }
 
 void OrthoContextMenu::callbackAddSpeaker()
@@ -337,7 +345,7 @@ void OrthoContextMenu::callbackAddSpeaker()
     ISoundShaderPtr soundShader;
 
     // If we have an active sound module, query the desired shader from the user
-    if (module::ModuleRegistry::Instance().moduleExists(MODULE_SOUNDMANAGER))
+    if (module::GlobalModuleRegistry().moduleExists(MODULE_SOUNDMANAGER))
     {
         IResourceChooser* chooser = GlobalDialogManager().createSoundShaderChooser();
 
@@ -362,30 +370,29 @@ void OrthoContextMenu::callbackAddSpeaker()
     try
     {
         // Create the speaker entity
-        scene::INodePtr spkNode = selection::algorithm::createEntityFromSelection(
+        auto spkNode = GlobalEntityModule().createEntityFromSelection(
             SPEAKER_CLASSNAME, _lastPoint
         );
 
         if (soundShader)
         {
             // Set the shader keyvalue
-            Entity* entity = Node_getEntity(spkNode);
-            assert(entity);
+            Entity& entity = spkNode->getEntity();
 
-            entity->setKeyValue("s_shader", soundShader->getName());
+            entity.setKeyValue("s_shader", soundShader->getName());
 
             // Initialise the speaker with suitable distance values
             SoundRadii radii = soundShader->getRadii();
 
-            entity->setKeyValue("s_mindistance", string::to_string(radii.getMin(true)));
-            entity->setKeyValue("s_maxdistance",
+            entity.setKeyValue("s_mindistance", string::to_string(radii.getMin(true)));
+            entity.setKeyValue("s_maxdistance",
                 (radii.getMax(true) > 0 ? string::to_string(radii.getMax(true)) : "10")
             );
         }
     }
-    catch (selection::algorithm::EntityCreationException&) 
+    catch (cmd::ExecutionFailure& e) 
     {
-        wxutil::Messagebox::ShowError(_("Unable to create speaker, classname not found."));
+        wxutil::Messagebox::ShowError(fmt::format(_("Unable to create speaker: {0}"), e.what()));
     }
 }
 
@@ -402,30 +409,31 @@ void OrthoContextMenu::callbackAddModel()
         ModelSelectorResult ms = ui::ModelSelector::chooseModel("", true, true);
 
         // If a model was selected, create the entity and set its model key
-        if (!ms.model.empty())
+        if (ms.model.empty())
         {
-            try {
-                scene::INodePtr modelNode = selection::algorithm::createEntityFromSelection(
-                    MODEL_CLASSNAME,
-                    _lastPoint
-                );
-
-                //Node_getTraversable(GlobalSceneGraph().root())->insert(modelNode);
-                Node_getEntity(modelNode)->setKeyValue("model", ms.model);
-                Node_getEntity(modelNode)->setKeyValue("skin", ms.skin);
-
-                // If 'createClip' is ticked, create a clip brush
-                if (ms.createClip)
-                {
-                    GlobalCommandSystem().execute("SurroundWithMonsterclip");
-                }
-            }
-            catch (selection::algorithm::EntityCreationException&)
-            {
-                wxutil::Messagebox::ShowError(_("Unable to create model, classname not found."));
-            }
+            return;
         }
 
+        try
+        {
+            auto modelNode = GlobalEntityModule().createEntityFromSelection(
+                MODEL_CLASSNAME, _lastPoint
+            );
+
+            //Node_getTraversable(GlobalSceneGraph().root())->insert(modelNode);
+            modelNode->getEntity().setKeyValue("model", ms.model);
+            modelNode->getEntity().setKeyValue("skin", ms.skin);
+
+            // If 'createClip' is ticked, create a clip brush
+            if (ms.createClip)
+            {
+                GlobalCommandSystem().execute("SurroundWithMonsterclip");
+            }
+        }
+        catch (cmd::ExecutionFailure& e)
+        {
+            wxutil::Messagebox::ShowError(fmt::format(_("Unable to create model: {0}"), e.what()));
+        }
     }
     else
     {
@@ -619,7 +627,7 @@ void OrthoContextMenu::addSectionItems(int section, bool noSpacer)
 
 const std::string& OrthoContextMenu::getName() const
 {
-    static std::string _name("OrthoContextMenu");
+    static std::string _name(MODULE_ORTHOCONTEXTMENU);
     return _name;
 }
 
@@ -632,15 +640,15 @@ const StringSet& OrthoContextMenu::getDependencies() const
         _dependencies.insert(MODULE_UIMANAGER);
         _dependencies.insert(MODULE_COMMANDSYSTEM);
         _dependencies.insert(MODULE_EVENTMANAGER);
-        _dependencies.insert(MODULE_RADIANT);
+        _dependencies.insert(MODULE_MAINFRAME);
     }
 
     return _dependencies;
 }
 
-void OrthoContextMenu::initialiseModule(const ApplicationContext& ctx)
+void OrthoContextMenu::initialiseModule(const IApplicationContext& ctx)
 {
-    GlobalRadiant().signal_radiantStarted().connect(
+    GlobalMainFrame().signal_MainFrameConstructed().connect(
         sigc::mem_fun(this, &OrthoContextMenu::constructMenu)
     );
 
