@@ -25,10 +25,11 @@
 #include <wx/radiobut.h>
 
 #include "PrefabPopulator.h"
-#include "map/algorithm/WorldspawnArgFinder.h"
+#include "entitylib.h"
 
 #include "string/trim.h"
 #include "string/case_conv.h"
+#include "util/ScopedBoolLock.h"
 #include <functional>
 
 namespace ui
@@ -63,7 +64,8 @@ PrefabSelector::PrefabSelector() :
     _useRecentPath(nullptr),
     _recentPathSelector(nullptr),
     _customPath(nullptr),
-	_insertAsGroupBox(nullptr)
+	_insertAsGroupBox(nullptr),
+	_handlingSelectionChange(false)
 {
 	SetSizer(new wxBoxSizer(wxVERTICAL));
 
@@ -286,10 +288,10 @@ PrefabSelector& PrefabSelector::Instance()
 		// Not yet instantiated, do it now
 		instancePtr.reset(new PrefabSelector);
 
-		// Register this instance with GlobalRadiant() at once
-		GlobalRadiant().signal_radiantShutdown().connect(
-			sigc::mem_fun(*instancePtr, &PrefabSelector::onRadiantShutdown)
-			);
+		// Pre-destruction cleanup
+		GlobalMainFrame().signal_MainFrameShuttingDown().connect(
+			sigc::mem_fun(*instancePtr, &PrefabSelector::onMainFrameShuttingDown)
+		);
 	}
 
 	return *instancePtr;
@@ -301,7 +303,7 @@ PrefabSelectorPtr& PrefabSelector::InstancePtr()
 	return _instancePtr;
 }
 
-void PrefabSelector::onRadiantShutdown()
+void PrefabSelector::onMainFrameShuttingDown()
 {
 	rMessage() << "PrefabSelector shutting down." << std::endl;
 
@@ -466,7 +468,14 @@ void PrefabSelector::clearPreview()
 
 void PrefabSelector::handleSelectionChange()
 {
-    wxDataViewItem item = _treeView->GetSelection();
+	if (_handlingSelectionChange)
+	{
+		return;
+	}
+
+	util::ScopedBoolLock lock(_handlingSelectionChange);
+	
+	wxDataViewItem item = _treeView->GetSelection();
 
     if (!item.IsOk())
     {
@@ -503,7 +512,7 @@ void PrefabSelector::handleSelectionChange()
 	if (_mapResource->load())
 	{
 		// Get the node from the resource
-        scene::IMapRootNodePtr root = _mapResource->getNode();
+        const auto& root = _mapResource->getRootNode();
 
 		assert(root);
 
@@ -539,13 +548,13 @@ void PrefabSelector::updateUsageInfo()
 {
 	std::string usage("");
 
-	if (_mapResource != NULL && _mapResource->getNode() != NULL)
+	if (_mapResource && _mapResource->getRootNode())
 	{
 		// Retrieve the root node
-		scene::INodePtr root = _mapResource->getNode();
+		const auto& root = _mapResource->getRootNode();
 
 		// Traverse the root to find the worldspawn
-		map::WorldspawnArgFinder finder("editor_description");
+		WorldspawnArgFinder finder("editor_description");
 		root->traverse(finder);
 
 		usage = finder.getFoundValue();
