@@ -91,6 +91,8 @@ private:
 
     std::list<std::string> _recentPaths;
 
+	bool _handlingSelectionChange;
+
 private:
 	// Private constructor, creates widgets
 	PrefabSelector();
@@ -127,8 +129,10 @@ private:
 	void onTreeStorePopulationFinished(wxutil::TreeModel::PopulationFinishedEvent& ev);
 	void onRescanPrefabs(wxCommandEvent& ev);
 
+	void onMainFrameShuttingDown();
+
 public:
-	int ShowModal();
+	int ShowModal() override;
 
 	/**
 	* Display the Selector instance, constructing it on first use, and
@@ -139,8 +143,6 @@ public:
 	* the dialog was closed.
 	*/
 	static Result ChoosePrefab(const std::string& curPrefab = "");
-
-	void onRadiantShutdown();
 };
 
 }
