@@ -3,11 +3,14 @@
 #include "i18n.h"
 #include "ieventmanager.h"
 #include "itextstream.h"
+#include "imainframe.h"
 #include "iuimanager.h"
+#include "icolourscheme.h"
 #include "igroupdialog.h"
 #include "iradiant.h"
 #include "ipreferencesystem.h"
 #include "imediabrowser.h"
+#include "ishaderclipboard.h"
 
 #include "wxutil/menu/IconTextMenuItem.h"
 #include "wxutil/GLWidget.h"
@@ -17,8 +20,6 @@
 
 #include "registry/registry.h"
 #include "shaderlib.h"
-#include "selection/algorithm/Shader.h"
-#include "selection/shaderclipboard/ShaderClipboard.h"
 
 #include "string/predicate.h"
 #include <functional>
@@ -31,6 +32,7 @@
 #include <wx/sizer.h>
 
 #include "string/case_conv.h"
+#include "debugging/gl.h"
 #include "TextureBrowserManager.h"
 
 namespace ui
@@ -346,12 +348,6 @@ TextureBrowser::TextureBrowser(wxWindow* parent) :
 TextureBrowser::~TextureBrowser()
 {
     GlobalTextureBrowser().unregisterTextureBrowser(this);
-
-    if (_textureToolbar != nullptr)
-    {
-        GlobalEventManager().disconnectToolbar(_textureToolbar);
-        _textureToolbar = nullptr;
-    }
 }
 
 void TextureBrowser::observeKey(const std::string& key)
@@ -606,13 +602,17 @@ void TextureBrowser::queueUpdate()
 
 void TextureBrowser::performUpdate()
 {
+    // During startup the openGL module might not have created the font yet
+    if (!GlobalMaterialManager().isRealised() || GlobalOpenGL().getFontHeight() == 0)
+    {
+        return;
+    }
+
     _updateNeeded = false;
 
     // Update all renderable items
     _tiles.clear();
 
-    if (!GlobalMaterialManager().isRealised()) return;
-
     CurrentPosition layout;
     _entireSpaceHeight = 0;
 
@@ -651,6 +651,11 @@ void TextureBrowser::onActiveShadersChanged()
 
 void TextureBrowser::focus(const std::string& name)
 {
+    if (name.empty())
+    {
+        return;
+    }
+
     for (const TextureTile& tile : _tiles)
     {
         // we have found when texdef->name and the shader name match
@@ -700,10 +705,10 @@ void TextureBrowser::selectTextureAt(int mx, int my)
         setSelectedShader(shader->getName());
 
         // Apply the shader to the current selection
-        selection::algorithm::applyShaderToSelection(shader->getName());
+        GlobalCommandSystem().executeCommand("SetShaderOnSelection", shader->getName());
 
 		// Copy the shader name to the clipboard too
-		GlobalShaderClipboard().setSource(shader->getName());
+		GlobalShaderClipboard().setSourceShader(shader->getName());
     }
 }
 
@@ -718,7 +723,7 @@ void TextureBrowser::draw()
 
     debug::assertNoGlErrors();
 
-    Vector3 colorBackground = ColourSchemes().getColour("texture_background");
+    Vector3 colorBackground = GlobalColourSchemeManager().getColour("texture_background");
     glClearColor(colorBackground[0], colorBackground[1], colorBackground[2], 0);
     glViewport(0, 0, _viewportSize.x(), _viewportSize.y());
     
@@ -943,8 +948,13 @@ void TextureBrowser::onIdle(wxIdleEvent& ev)
     }
 }
 
-void TextureBrowser::onRender()
+bool TextureBrowser::onRender()
 {
+    if (!GlobalMainFrame().screenUpdatesEnabled())
+    {
+        return false;
+    }
+
 	debug::assertNoGlErrors();
 
     if (_updateNeeded)
@@ -955,6 +965,8 @@ void TextureBrowser::onRender()
     draw();
 
     debug::assertNoGlErrors();
+
+    return true;
 }
 
 } // namespace ui
