@@ -68,7 +68,6 @@ public:
 	bool showAxes() const;
 	bool showWorkzone() const;
 	bool showSizeInfo() const;
-	bool higherEntitySelectionPriority() const;
 
 	unsigned int defaultBlockSize() const;
 
@@ -104,6 +103,7 @@ public:
 
 	// Sets the origin of all available views
 	void setOrigin(const Vector3& origin) override;
+	Vector3 getActiveViewOrigin() override;
 
 	// Sets the scale of all available views
 	void setScale(float scale) override;
@@ -120,6 +120,9 @@ public:
 	EViewType getActiveViewType() const override;
 	void setActiveViewType(EViewType viewType) override;
 
+	IOrthoView& getActiveView() override;
+	IOrthoView& getViewByType(EViewType viewType) override;
+
 	void toggleActiveView(const cmd::ArgumentList& args);
 
 	// Retrieves the pointer to the first view matching the given view type
@@ -159,7 +162,7 @@ public:
 	// RegisterableModule implementation
 	const std::string& getName() const override;
 	const StringSet& getDependencies() const override;
-	void initialiseModule(const ApplicationContext& ctx) override;
+	void initialiseModule(const IApplicationContext& ctx) override;
 	void shutdownModule() override;
 
 private:
