@@ -5,8 +5,10 @@
 #include "iundo.h"
 #include "ieventmanager.h"
 #include "imainframe.h"
+#include "icolourscheme.h"
 #include "ientity.h"
 #include "igrid.h"
+#include "iregion.h"
 #include "iuimanager.h"
 
 #include "wxutil/MouseButton.h"
@@ -14,30 +16,27 @@
 #include "string/string.h"
 #include "selectionlib.h"
 
-#include "brush/TexDef.h"
 #include "ibrush.h"
-#include "camera/GlobalCamera.h"
 #include "camera/CameraSettings.h"
 #include "ui/ortho/OrthoContextMenu.h"
 #include "ui/overlay/Overlay.h"
 #include "ui/texturebrowser/TextureBrowser.h"
-#include "map/RegionManager.h"
-#include "map/Map.h"
-#include "selection/algorithm/General.h"
-#include "selection/algorithm/Primitives.h"
 #include "registry/registry.h"
 #include "selection/Device.h"
-#include "selection/SelectionTest.h"
+#include "selection/SelectionVolume.h"
 #include "util/ScopedBoolLock.h"
+#include "debugging/gl.h"
 
 #include "GlobalXYWnd.h"
 #include "XYRenderer.h"
 #include "gamelib.h"
 #include "scenelib.h"
 #include "maplib.h"
-#include "render/frontend/RenderableCollectionWalker.h"
+#include "render/RenderableCollectionWalker.h"
 
+#include <wx/frame.h>
 #include <fmt/format.h>
+#include <sigc++/functors/mem_fun.h>
 #include <functional>
 
 namespace ui
@@ -138,7 +137,8 @@ XYWnd::XYWnd(int id, wxWindow* parent) :
     GlobalSceneGraph().addSceneObserver(this);
 
     // greebo: Connect <self> as CameraObserver to the CamWindow. This way this class gets notified on camera change
-    GlobalCamera().addCameraObserver(this);
+    _sigCameraChanged = GlobalCameraManager().signal_cameraChanged().connect(
+        sigc::mem_fun(this, &XYWnd::onCameraMoved));
 }
 
 // Destructor
@@ -165,9 +165,7 @@ void XYWnd::destroyXYView()
     // Remove <self> from the scene change callback list
     GlobalSceneGraph().removeSceneObserver(this);
 
-    // greebo: Remove <self> as CameraObserver from the CamWindow.
-    GlobalCamera().removeCameraObserver(this);
-
+    _sigCameraChanged.disconnect();
     _wxGLWidget = nullptr;
 }
 
@@ -291,13 +289,15 @@ bool XYWnd::isActive() const {
     return _isActive;
 };
 
-const Vector3& XYWnd::getOrigin() {
+const Vector3& XYWnd::getOrigin() const 
+{
     return _origin;
 }
 
 void XYWnd::setOrigin(const Vector3& origin) {
     _origin = origin;
     updateModelview();
+    queueDraw();
 }
 
 void XYWnd::scroll(int x, int y)
@@ -431,9 +431,10 @@ void XYWnd::setCursorType(CursorType type)
     };
 }
 
-// Callback that gets invoked on camera move
-void XYWnd::cameraMoved() {
-    if (GlobalXYWnd().camXYUpdate()) {
+void XYWnd::onCameraMoved()
+{
+    if (GlobalXYWnd().camXYUpdate())
+    {
         queueDraw();
     }
 }
@@ -603,9 +604,10 @@ Vector4 XYWnd::getWindowCoordinates() {
     double h = (_height / 2 / _scale);
 
     // Query the region minimum/maximum vectors
-    Vector3 regionMin;
-    Vector3 regionMax;
-    GlobalRegion().getMinMax(regionMin, regionMax);
+    auto regionBounds = GlobalRegionManager().getRegionBounds();
+
+    Vector3 regionMin = regionBounds.origin - regionBounds.extents;
+    Vector3 regionMax = regionBounds.origin + regionBounds.extents;
 
     double xb = _origin[nDim1] - w;
     // Constrain this value to the region minimum
@@ -681,9 +683,9 @@ void XYWnd::drawGrid()
 
     if (GlobalXYWnd().showGrid())
     {
-        Vector3 colourGridBack = ColourSchemes().getColour("grid_background");
-        Vector3 colourGridMinor = ColourSchemes().getColour("grid_minor");
-        Vector3 colourGridMajor = ColourSchemes().getColour("grid_major");
+        Vector3 colourGridBack = GlobalColourSchemeManager().getColour("grid_background");
+        Vector3 colourGridMinor = GlobalColourSchemeManager().getColour("grid_minor");
+        Vector3 colourGridMajor = GlobalColourSchemeManager().getColour("grid_major");
 
         // run grid rendering twice, first run is minor grid, then major
         // NOTE: with a bit more work, we can have variable number of grids
@@ -842,7 +844,7 @@ void XYWnd::drawGrid()
     {
         char text[32];
 
-        glColor3dv(ColourSchemes().getColour("grid_text"));
+        glColor3dv(GlobalColourSchemeManager().getColour("grid_text"));
 
         double offx = _origin[nDim2] + h - 12 / _scale;
         double offy = _origin[nDim1] - w + 1 / _scale;
@@ -863,7 +865,7 @@ void XYWnd::drawGrid()
 
         if (isActive())
         {
-            glColor3dv(ColourSchemes().getColour("active_view_name"));
+            glColor3dv(GlobalColourSchemeManager().getColour("active_view_name"));
         }
 
         // we do this part (the old way) only if show_axis is disabled
@@ -881,8 +883,8 @@ void XYWnd::drawGrid()
 
         const std::string colourNameX = (_viewType == YZ) ? "axis_y" : "axis_x";
         const std::string colourNameY = (_viewType == XY) ? "axis_y" : "axis_z";
-        const Vector3& colourX = ColourSchemes().getColour(colourNameX);
-        const Vector3& colourY = ColourSchemes().getColour(colourNameY);
+        const Vector3& colourX = GlobalColourSchemeManager().getColour(colourNameX);
+        const Vector3& colourY = GlobalColourSchemeManager().getColour(colourNameY);
 
         // draw two lines with corresponding axis colors to highlight current view
         // horizontal line: nDim1 color
@@ -922,7 +924,7 @@ void XYWnd::drawGrid()
 
         if (wz.bounds.isValid())
         {
-            glColor3dv( ColourSchemes().getColour("workzone") );
+            glColor3dv(GlobalColourSchemeManager().getColour("workzone"));
             glBegin( GL_LINES );
 
             glVertex2f( xb, wz.min[nDim2] );
@@ -974,7 +976,7 @@ void XYWnd::drawBlockGrid()
 
     // draw major blocks
 
-    glColor3dv(ColourSchemes().getColour("grid_block"));
+    glColor3dv(GlobalColourSchemeManager().getColour("grid_block"));
     glLineWidth (2);
 
     glBegin (GL_LINES);
@@ -1008,45 +1010,57 @@ void XYWnd::drawBlockGrid()
     glColor4f(0, 0, 0, 0);
 }
 
-void XYWnd::drawCameraIcon(const Vector3& origin, const Vector3& angles)
+void XYWnd::drawCameraIcon()
 {
-    float   x, y, fov, box;
-    float a;
+    try
+    {
+        auto& camera = GlobalCameraManager().getActiveView();
 
-    fov = 48 / _scale;
-    box = 16 / _scale;
+        const auto& origin = camera.getCameraOrigin();
+        const auto& angles = camera.getCameraAngles();
 
-    if (_viewType == XY) {
-        x = origin[0];
-        y = origin[1];
-        a = degrees_to_radians(angles[CAMERA_YAW]);
+        float x, y, fov, box;
+        float a;
+
+        fov = 48 / _scale;
+        box = 16 / _scale;
+
+        if (_viewType == XY) {
+            x = origin[0];
+            y = origin[1];
+            a = degrees_to_radians(angles[camera::CAMERA_YAW]);
+        }
+        else if (_viewType == YZ) {
+            x = origin[1];
+            y = origin[2];
+            a = degrees_to_radians(angles[camera::CAMERA_PITCH]);
+        }
+        else {
+            x = origin[0];
+            y = origin[2];
+            a = degrees_to_radians(angles[camera::CAMERA_PITCH]);
+        }
+
+        glColor3dv(GlobalColourSchemeManager().getColour("camera_icon"));
+        glBegin(GL_LINE_STRIP);
+        glVertex3f(x - box, y, 0);
+        glVertex3f(x, y + (box / 2), 0);
+        glVertex3f(x + box, y, 0);
+        glVertex3f(x, y - (box / 2), 0);
+        glVertex3f(x - box, y, 0);
+        glVertex3f(x + box, y, 0);
+        glEnd();
+
+        glBegin(GL_LINE_STRIP);
+        glVertex3f(x + static_cast<float>(fov * cos(a + c_pi / 4)), y + static_cast<float>(fov * sin(a + c_pi / 4)), 0);
+        glVertex3f(x, y, 0);
+        glVertex3f(x + static_cast<float>(fov * cos(a - c_pi / 4)), y + static_cast<float>(fov * sin(a - c_pi / 4)), 0);
+        glEnd();
     }
-    else if (_viewType == YZ) {
-        x = origin[1];
-        y = origin[2];
-        a = degrees_to_radians(angles[CAMERA_PITCH]);
+    catch (const std::runtime_error&)
+    {
+        return; // no camera present
     }
-    else {
-        x = origin[0];
-        y = origin[2];
-        a = degrees_to_radians(angles[CAMERA_PITCH]);
-    }
-
-    glColor3dv(ColourSchemes().getColour("camera_icon"));
-    glBegin(GL_LINE_STRIP);
-    glVertex3f (x-box,y,0);
-    glVertex3f (x,y+(box/2),0);
-    glVertex3f (x+box,y,0);
-    glVertex3f (x,y-(box/2),0);
-    glVertex3f (x-box,y,0);
-    glVertex3f (x+box,y,0);
-    glEnd();
-
-    glBegin(GL_LINE_STRIP);
-    glVertex3f (x + static_cast<float>(fov*cos(a+c_pi/4)), y + static_cast<float>(fov*sin(a+c_pi/4)), 0);
-    glVertex3f (x, y, 0);
-    glVertex3f (x + static_cast<float>(fov*cos(a-c_pi/4)), y + static_cast<float>(fov*sin(a-c_pi/4)), 0);
-    glEnd();
 }
 
 // can be greatly simplified but per usual i am in a hurry
@@ -1062,7 +1076,7 @@ void XYWnd::drawSizeInfo(int nDim1, int nDim2, const Vector3& vMinBounds, const
 
   Vector3 vSize(vMaxBounds - vMinBounds);
 
-  glColor3dv(ColourSchemes().getColour("brush_size_info"));
+  glColor3dv(GlobalColourSchemeManager().getColour("brush_size_info"));
 
   std::ostringstream dimensions;
 
@@ -1200,7 +1214,7 @@ void XYWnd::updateProjection() {
 
     _projection[15] = 1.0f;
 
-    _view.Construct(_projection, _modelView, _width, _height);
+    _view.construct(_projection, _modelView, _width, _height);
 }
 
 // note: modelview matrix must have a uniform scale, otherwise strange things happen when rendering the rotation manipulator.
@@ -1259,14 +1273,14 @@ void XYWnd::updateModelview() {
     _modelView[3] = _modelView[7] = _modelView[11] = 0;
     _modelView[15] = 1;
 
-    _view.Construct(_projection, _modelView, _width, _height);
+    _view.construct(_projection, _modelView, _width, _height);
 }
 
 void XYWnd::draw()
 {
     // clear
     glViewport(0, 0, _width, _height);
-    Vector3 colourGridBack = ColourSchemes().getColour("grid_background");
+    Vector3 colourGridBack = GlobalColourSchemeManager().getColour("grid_background");
     glClearColor (colourGridBack[0], colourGridBack[1], colourGridBack[2], 0);
 
     glClear(GL_COLOR_BUFFER_BIT);
@@ -1372,7 +1386,7 @@ void XYWnd::draw()
     }
 
     if (xyWndManager.showCrossHairs()) {
-        Vector3 colour = ColourSchemes().getColour("xyview_crosshairs");
+        Vector3 colour = GlobalColourSchemeManager().getColour("xyview_crosshairs");
         glColor4d(colour[0], colour[1], colour[2], 0.8f);
         glBegin (GL_LINES);
         if (_viewType == XY) {
@@ -1398,7 +1412,7 @@ void XYWnd::draw()
 
     // greebo: Draw the clipper's control points
     {
-        glColor3dv(ColourSchemes().getColour("clipper"));
+        glColor3dv(GlobalColourSchemeManager().getColour("clipper"));
         glPointSize(4);
 
         if (GlobalClipper().clipMode()) {
@@ -1415,11 +1429,7 @@ void XYWnd::draw()
     glScalef(_scale, _scale, 1);
     glTranslatef(-_origin[nDim1], -_origin[nDim2], 0);
 
-    CamWndPtr cam = GlobalCamera().getActiveCamWnd();
-
-    if (cam != NULL) {
-        drawCameraIcon(cam->getCameraOrigin(), cam->getCameraAngles());
-    }
+    drawCameraIcon();
 
     if (!_activeMouseTools.empty())
     {
@@ -1447,13 +1457,13 @@ void XYWnd::draw()
 
             switch (_viewType) {
                 case YZ:
-                    glColor3dv(ColourSchemes().getColour("axis_x"));
+                    glColor3dv(GlobalColourSchemeManager().getColour("axis_x"));
                     break;
                 case XZ:
-                    glColor3dv(ColourSchemes().getColour("axis_y"));
+                    glColor3dv(GlobalColourSchemeManager().getColour("axis_y"));
                     break;
                 case XY:
-                    glColor3dv(ColourSchemes().getColour("axis_z"));
+                    glColor3dv(GlobalColourSchemeManager().getColour("axis_z"));
                     break;
             }
 
@@ -1542,14 +1552,19 @@ void XYWnd::onGLResize(wxSizeEvent& ev)
 	ev.Skip();
 }
 
-void XYWnd::onRender()
+bool XYWnd::onRender()
 {
 	if (GlobalMainFrame().screenUpdatesEnabled())
 	{
         util::ScopedBoolLock drawLock(_drawing);
 
 		draw();
+
+        return true;
 	}
+    
+
+    return false; // nothing rendered
 }
 
 void XYWnd::onGLWindowScroll(wxMouseEvent& ev)
